(ns wanijo.attribute.routes
  (:require [compojure.core :refer [defroutes GET POST DELETE]]
            [ring.util.response :as resp]
            [wanijo.framework.form :as form]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.attribute.domain :as domain]
            [wanijo.schema.view :as view-schema]
            [wanijo.schema.domain :as domain-schema]
            [wanijo.schema.routes :as routes-schema]))

(defn new! [req]
  (let [schema-uuid (get-in req [:params :schema])]
    (if (form/valid? view-schema/attr-form req)
      (do
        (domain/create-new!
         (:params req)
         schema-uuid
         (get-in req [:session :uuid]))
        (resp/redirect (path :schema-show {:uuid schema-uuid})))
      (routes-schema/view! schema-uuid req))))

(defn edit! [schema-uuid req]
  (if (form/valid? view-schema/attr-form req)
    (do
      (domain/edit! (:params req))
      (resp/redirect (path :schema-show {:uuid schema-uuid})))
    (routes-schema/view! schema-uuid req)))

(defn delete! [uuid req]
  (domain/delete-by-uuid! uuid)
  (resp/redirect (path :schema-show {:uuid (get-in req [:params :schema])})))

(defroutes routes
  (POST (register! :attribute-new "/attribute/new")
        []
        new!)
  (POST (register! :attribute-edit "/attribute/edit/:schema")
        [schema :as req]
        (edit! schema req))
  (DELETE (register! :attribute-delete "/attribute/:uuid/delete")
          [uuid :as req]
          (delete! uuid req)))
