(ns wanijo.framework.view
  (:require [hiccup.page :refer
             [html5 include-css include-js]]
            [hiccup.form :as hform]
            [wanijo.framework.routing :refer [path]]))

(defn btnlink
  ([target caption]
   (btnlink target caption ""))
  ([target caption clss]
   [:a {:href target}
    [:button {:class clss}
     caption]]))

(defn delete-btn []
  (hform/submit-button {:class "delete-btn"}
                       "☒ Delete!"))

(defn layout!
  [& {:keys [content title session]
      :or {content []
           title nil
           session {}}}]
  (let [ident (:ident session)
        authed? (some? ident)
        devmode? (:devmode session)]
    (html5
     [:head
      [:meta {:charset "utf-8"}]
      [:meta {:name "viewport"
              :content "width=device-width,initial-scale=1,shrink-to-fit=no"}]
      [:title (str "wan ijo" (when title (str " - " title)))]
      (include-css "/css/app.css")
      (when devmode? (include-css "/css/devmode.css"))
      (include-js "/js/frontend.js")]
     [:body
      [:section.grid
       [:header
        [:h1.app-title "wan ijo"
         (when authed?
           [:small.app-title__hello
            "Hi, "
            [:a {:href (path :user-profile)} ident]])]
        (when authed?
          [:section.header-content
           (btnlink (path :schema-overview)
                    "▤ Schemas"
                    "header-content__link")
           (btnlink (path :auth-logout)
                    "Logout!"
                    "header-content__link")])]
       [:nav
        (when authed?
          [:section.schemas
           [:h2 "▤ Created Schemas"]
           [:ul
            (for [schema (:created-schemas session)]
              [:li (:name schema)])]
           [:h2 "▤ Other Schemas"]
           [:ul
            (for [schema (:other-schemas session)]
              [:li (:name schema)])]])]
       (into [:main] content)
       [:aside (when authed? "aside")]
       [:footer
        [:small "Ilo pali e ijo"]]]])))

(defn flash-error [content]
  [:section.flash--error
   [:h2.flash__heading--error "Warning"]
   content])
