(ns wanijo.schema.routes
  (:require [compojure.core :refer [defroutes GET POST DELETE]]
            [ring.util.response :as resp]
            [wanijo.framework.view :as view]
            [wanijo.framework.form :as form]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.schema.domain :as domain]
            [wanijo.user.domain :as domain-user]
            [wanijo.schema.view :as view-schema]
            [wanijo.attribute.domain :as domain-attr]))

(defn new! [req]
  (if (form/valid? view-schema/form req)
    (do
      (domain/create-new!
       (get-in req [:params :name])
       (get-in req [:session :uuid]))
      (resp/redirect (path :schema-overview)))
    (view-schema/overview! req)))

(defn delete-schema! [uuid session]
  (if (domain/can-user-modify? uuid (:uuid session))
    (do
      (domain/delete! uuid)
      (resp/redirect (path :schema-overview)))
    {:status 403}))

(defn view! [uuid req]
  (view-schema/show-schema!
   (domain/find-with-assigned-users! uuid)
   (domain-attr/find-by-schema! uuid)
   (assoc-in view-schema/assign-form
             [:fields :assigned :options]
             (map #(vector (:ident %) (:uuid %))
                  (domain-user/all!)))
   req))

(defn edit! [req]
  (let [uuid (get-in req [:params :uuid])]
    (if (form/valid? view-schema/form req)
      (do
        (domain/edit! (:params req))
        (resp/redirect (path :schema-show (:params req))))
      (view! uuid req))))

(defn assign! [req]
  (let [uuid (get-in req [:params :uuid])
        users (get-in req [:params :assigned])]
    (println (:params req))
    (if (form/valid? view-schema/assign-form req)
      (do
        (domain/assign-users! uuid
                              (if (vector? users)
                                users
                                [users]))
        (resp/redirect (path :schema-show (:params req))))
      (view! uuid req))))

(defroutes routes
  (GET (register! :schema-overview "/schema")
       []
       view-schema/overview!)
  (GET (register! :schema-show "/schema/:uuid")
       [uuid :as req]
       (view! uuid req))
  (POST (register! :schema-new "/schema/new")
        []
        new!)
  (POST (register! :schema-edit "/schema/edit")
        []
        edit!)
  (POST (register! :schema-assign "/schema/assign")
        []
        assign!)
  (DELETE (register! :schema-delete "/schema/:uuid")
          [uuid :as req]
          (delete-schema! uuid (:session req))))
