(ns wanijo.schema.view
  (:require [hiccup.form :as hform]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [wanijo.framework.view :as view]
            [wanijo.framework.form :as form]
            [wanijo.framework.routing :refer [path]]
            [wanijo.framework.time :refer [prettify-dt]]
            [wanijo.schema.domain :as domain]
            [wanijo.attribute.domain :as attr-domain]))

(def form
  {:fields {:name {:label "Name"
                   :required true
                   :spec ::domain/name}}})

(def attr-form
  {:fields {:name {:label "Name"
                   :required true
                   :spec ::attr-domain/name}
            :type {:label "Type"
                   :required true
                   :spec ::attr-domain/type
                   :options [["String" "string"]
                             ["Text" "text"]
                             ["Date" "date"]
                             ["File" "file"]]}
            :required {:label "Required"
                       :required false
                       :spec ::attr-domain/required
                       :widget :checkbox}}})

(def assign-form
  {:fields {:assigned {:label "Users"
                       :required false
                       :spec ::domain/assigned-to
                       :widget :mselect}}})

(defn overview! [req]
  (let [session (:session req)
        uuid (:uuid session)
        schemas (domain/all-created-by! uuid)]
    (view/layout!
     :session session
     :content
     [[:h1 "All schemas"]
      [:table
       [:thead
        [:tr
         [:th "Name"]
         [:th "Created"]]]
       [:tbody
        (for [schema schemas]
          [:tr
           [:td
            [:a {:href (path :schema-show schema)}
             (:name schema)]]
           [:td
            (prettify-dt (:created_at schema))]])]]
      [:h1 "New schema"]
      (hform/form-to [:post (path :schema-new)]
                     (form/render-widgets form {} req)
                     (hform/submit-button "Create"))])))

(defn show-schema! [schema attrs assign-form req]
  (view/layout!
   :session (:session req)
   :content
   [[:h1 "Schema "
     [:span.schema-title__name (:name schema)]]
    [:h2 "Edit"]
    (hform/form-to [:post (path :schema-edit)]
                   (form/render-widgets form schema req)
                   (hform/hidden-field "uuid" (:uuid schema))
                   (hform/submit-button "Edit"))
    [:h3 "Assign to users"]
    (hform/form-to [:post (path :schema-assign)]
                   (form/render-widgets assign-form schema req)
                   (hform/hidden-field "uuid" (:uuid schema))
                   (hform/submit-button "Assign"))
    [:h2 "Attributes"]
    [:ul.schema-attributes
     (for [attr attrs]
       [:li
        (hform/form-to [:post (path :attribute-edit {:schema (:uuid schema)})]
                       (form/render-widgets attr-form attr req)
                       (hform/hidden-field "uuid" (:uuid attr))
                       (hform/submit-button "Save"))
        (hform/form-to [:delete (path :attribute-delete attr)]
                       (anti-forgery-field)
                       (hform/hidden-field "schema" (:uuid schema))
                       (hform/submit-button {:class "delete-btn"} "Delete!")
                       [:label "Created at "
                        [:em (prettify-dt (:created_at attr))]])])]
    [:h3 "New attribute"]
    (hform/form-to [:post (path :attribute-new)]
                   (form/render-widgets attr-form {} req)
                   (hform/hidden-field "schema" (:uuid schema))
                   (hform/submit-button "Create"))
    [:h2 "Actions"]
    (hform/form-to {:class "inline"}
                   [:delete (path :schema-delete schema)]
                   (anti-forgery-field)
                   (view/delete-btn))]))
